package cn.kdan.pdf.tech.core.service.impl;

import cn.kdan.pdf.tech.core.constant.VppDeviceConstant;
import cn.kdan.pdf.tech.core.enums.CaptchaActionEnum;
import cn.kdan.pdf.tech.core.enums.DeviceStatusEnum;
import cn.kdan.pdf.tech.core.enums.LicenseCodeStatusEnum;
import cn.kdan.pdf.tech.core.enums.PlatformEnum;
import cn.kdan.pdf.tech.core.mapper.AppsMapper;
import cn.kdan.pdf.tech.core.mapper.ext.ExtVppDeviceMapper;
import cn.kdan.pdf.tech.core.model.*;
import cn.kdan.pdf.tech.core.params.VerifyParam;
import cn.kdan.pdf.tech.core.pojo.dto.VppPrivateDeployment;
import cn.kdan.pdf.tech.core.pojo.vo.*;
import cn.kdan.pdf.tech.core.service.*;
import cn.kdan.pdf.tech.core.utils.ThreadPoolSingleUtil;
import com.alibaba.fastjson.JSON;
import com.github.pagehelper.PageHelper;
import com.github.pagehelper.PageInfo;
import constant.CommonConstant;
import exception.BackendRuntimeException;
import lombok.extern.slf4j.Slf4j;
import org.apache.commons.codec.digest.DigestUtils;
import org.apache.commons.lang.StringUtils;
import org.omg.PortableInterceptor.ACTIVE;
import org.springframework.beans.BeanUtils;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.beans.factory.annotation.Value;
import org.springframework.stereotype.Service;
import org.springframework.transaction.annotation.Transactional;
import org.springframework.util.CollectionUtils;
import org.springframework.util.ObjectUtils;
//import cn.kdan.pdf.tech.core.utils.EmailUtils;

import javax.annotation.Resource;
import java.util.*;
import java.util.stream.Collectors;

import static cn.hutool.core.bean.BeanUtil.copyProperties;


/**
 * 用户与团队的关系类
 *
 * @author tangxiangan
 */
@Slf4j
@Service
public class VppDeviceServiceImpl implements VppDeviceService {

    @Resource
    private ExtVppDeviceMapper extVppDeviceMapper;
    @Resource
    private VppMemberService memberService;
    @Resource
    private ProductService productService;
//    @Autowired
//    private EmailUtils emailUtils;
    @Resource
    private SubscriptionService subscriptionService;
    @Resource
    private VppRTeamMemberRoleService vppRTeamMemberRoleService;
    @Resource
    private VppLicenseCodeService licenseCodeService;
    @Resource
    private AppsMapper appsMapper;
    // 运营后台地址，图片地址emailImagesUrl
    @Value("${readerproUrl}")
    private String readerproUrl;
    @Value("${emailImagesUrl}")
    private String emailImagesUrl;

    @Override
    public PageInfo<VppDeviceVO> page(Integer page, Integer pageSize, Integer productId, Integer status, String teamId, String queryString) {
        VppMember member = memberService.getCurrentUser();
        List<String> teamIds = new ArrayList<>();
        if(ObjectUtils.isEmpty(teamId)){
            List<VppRTeamMemberRoleVO> list = vppRTeamMemberRoleService.listByMemberIdAdmin(member.getId());
            List<String> idList  = list.stream().map(VppRTeamMemberRoleVO::getTeamId).collect(Collectors.toList());
            teamIds = idList;
        }else{
            teamIds.add(teamId);
        }
        if(CollectionUtils.isEmpty(teamIds)){
            throw new BackendRuntimeException(VppDeviceConstant.EXCEPTION_MSG_NO_TEAM);
        }
        PageHelper.startPage(page, pageSize);
        return new PageInfo<>(extVppDeviceMapper.listWithInfo(teamIds, productId, status, queryString));
    }

    @Override
    public List<VppDeviceVO> listWithInfo(Integer productId, Integer status, String teamId, String queryString) {
        VppMember member = memberService.getCurrentUser();
        memberService.getCurrentUser();
        List<String> teamIds = new ArrayList<>();
        if(ObjectUtils.isEmpty(teamId)){
            List<VppRTeamMemberRoleVO> list = vppRTeamMemberRoleService.listByMemberIdAdmin(member.getId());
            List<String> idList  = list.stream().map(VppRTeamMemberRoleVO::getTeamId).collect(Collectors.toList());
            teamIds = idList;
        }else{
            teamIds.add(teamId);
        }
        if(CollectionUtils.isEmpty(teamIds)){
            throw new BackendRuntimeException(VppDeviceConstant.EXCEPTION_MSG_NO_TEAM);
        }
        return extVppDeviceMapper.listWithInfo(teamIds, productId, status, queryString);
    }


    @Override
    public void unBindByMemberId(String memberId, List<String> cdKeys) {
        //查出用户所有已经激活的设备
        DevicesExample example = new DevicesExample();
        DevicesExample.Criteria criteria = example.createCriteria().andStatusEqualTo(DeviceStatusEnum.ACTIVE.code());
        if (!CollectionUtils.isEmpty(cdKeys)) {
            criteria.andCdkeyIn(cdKeys);
        }
        //修改设备为未授权
        Devices devices = new Devices();
        devices.setStatus(DeviceStatusEnum.UNACTIVATED.code());
        devices.setSubscriptionId(null);
        devices.setEndUpAt(null);
        devices.setCanceledDate(new Date());
        extVppDeviceMapper.updateByExampleSelective(devices, example);
    }

    @Transactional
    @Override
    public void cancel(String deviceId) {
        Devices devices = extVppDeviceMapper.selectByPrimaryKey(deviceId);
        if (ObjectUtils.isEmpty(devices)) {
            throw new BackendRuntimeException(VppDeviceConstant.EXCEPTION_MSG_DEVICE_NOT_EXIST);
        }

        //设置序列码状态
        String cdkey = devices.getCdkey();
        String model = devices.getModel();
        LicenseCodes code = licenseCodeService.getByCdKey(cdkey);
        VppMember vppMember = memberService.getById(code.getVppMemberId());

        //更新设备状态
        devices.setStatus(DeviceStatusEnum.UNACTIVATED.code());
        devices.setCanceledDate(new Date());
        devices.setSubscriptionId(null);
        devices.setEndUpAt(null);
        devices.setCdkey(null);
        log.info("update device:" + devices);
        extVppDeviceMapper.updateByPrimaryKeySelective(devices);
        // 获取序列码下所有已激活设备
        DevicesExample devicesExample = new DevicesExample();
        devicesExample.createCriteria().andCdkeyEqualTo(cdkey).andStatusEqualTo(DeviceStatusEnum.ACTIVE.code());
        long countByExample = extVppDeviceMapper.countByExample(devicesExample);
        code.setUsedTimes((short) countByExample);
        Integer code1 = countByExample == 0 ? LicenseCodeStatusEnum.ASSIGNED.code() : LicenseCodeStatusEnum.PARTIALLY_ACTIVATED.code();
        code.setValidFlag(code1);
        code.setUpdatedAt(new Date());
        log.info("update code:" + code);
        licenseCodeService.update(code);

//        log.info("send cancel email!");
//        ThreadPoolSingleUtil.getInstance().excute(() -> {
//            if(!ObjectUtils.isEmpty(vppMember)){
//                Subscriptions subscriptions = subscriptionService.getById(code.getSubscriptionId());
//                Products product = productService.getById(subscriptions.getProductId());
//                String helpLink = vppLicenseCodeServiceImpl.getHelpLink(product.getCode());
//                String devicesModel = StringUtils.isNotEmpty(model) ? model : "";
//                String text = String.format(CaptchaActionEnum.CANCEL_DEVICE_ASSIGN.text(),vppMember.getEmail(),devicesModel,product.getName(),product.getName(),devicesModel, helpLink);
//                // 替换图片动态地址和取消订阅链接
//                text = text.replace("emailImagesUrl", emailImagesUrl).replace("cancelEmailUrl", readerproUrl + "/unsubscribe?email=" + vppMember.getEmail());
////                emailUtils.sendMail(CaptchaActionEnum.CANCEL_DEVICE_ASSIGN.subject(), text, Collections.singletonList(vppMember.getEmail()));
//            }
//        });
    }

    /**
     * 获取序列激活的设备
     * @param cdKeyList 序列码
     * @return
     */
    @Override
    public List<Devices> getByCdKeyList(List<String> cdKeyList) {
        DevicesExample devicesExample = new DevicesExample();
        devicesExample.createCriteria().andCdkeyIn(cdKeyList).andStatusEqualTo(1);
        return extVppDeviceMapper.selectByExample(devicesExample);
    }

    @Override
    public ActivationVO activation(VerifyParam verifyParam) {
        ActivationVO activationVO = new ActivationVO();
        // 查询序列码信息
        List<LicenseCodesVO> licenseCodes = licenseCodeService.getWithInfo(verifyParam.getSubscription().getCdkey());
        if (CollectionUtils.isEmpty(licenseCodes)) {
            throw new BackendRuntimeException(VppDeviceConstant.EXCEPTION_MSG_INVALID_LICENSE_KEY);
        }
        LicenseCodesVO licenseCodesVO = licenseCodes.get(0);

        //验证序列码是否合法
        String sign = licenseCodesVO.getSign();
        VppPrivateDeployment deployment = new VppPrivateDeployment();
        BeanUtils.copyProperties(licenseCodesVO, deployment);
        //将对象转化为json
        String deploymentJson = JSON.toJSONString(deployment) + "vpp_private_deployment";
        //对上述json进行MD5加密并且和sign对比
        String signOriginal = DigestUtils.md5Hex(deploymentJson);
        if (!signOriginal.equals(sign)) {
            throw new BackendRuntimeException(VppDeviceConstant.EXCEPTION_MSG_INVALID_LICENSE_KEY);
        }

        // 定义设备状态集合，用于判断设备是否已激活
        Set<Integer> activeStatuses = new HashSet<>();
        activeStatuses.add(DeviceStatusEnum.UNACTIVATED.code());
        activeStatuses.add(DeviceStatusEnum.ACTIVE.code());

        // 查询设备是否已激活
        DevicesExample devicesExample = new DevicesExample();
        devicesExample.createCriteria()
                .andUniqueSnEqualTo(verifyParam.getDevice().getUniqueSn())
                .andCdkeyEqualTo(verifyParam.getSubscription().getCdkey())
                .andStatusIn(new ArrayList<>(activeStatuses));
        List<Devices> list = extVppDeviceMapper.selectByExample(devicesExample);

        if (!CollectionUtils.isEmpty(list)) {
            // 设备已激活，返回设备信息
            Devices devices = list.get(0);
            DeviceResVO deviceResVO = new DeviceResVO();
            deviceResVO.setStatus(Objects.requireNonNull(DeviceStatusEnum.getEnumByCode(devices.getStatus())).value());
            Products products = productService.getById(licenseCodesVO.getProductId());
            deviceResVO.setProductCode(products.getCode());
            activationVO.setDevice(deviceResVO);
            return activationVO;
        }

        // 验证序列码的应用程序代码和激活请求的应用程序代码是否一致
        if (!Objects.equals(licenseCodesVO.getAppCode(), verifyParam.getSubscription().getAppCode())) {
            throw new BackendRuntimeException(VppDeviceConstant.EXCEPTION_MSG_LICENSE_NOT_MATCH_PRODUCT);
        }

        // 验证序列码使用次数是否超过限制
        if (licenseCodesVO.getUsedTimes() >= licenseCodesVO.getTimes()) {
            throw new BackendRuntimeException(VppDeviceConstant.EXCEPTION_MSG_MAX_DEVICE_NUM);
        }

        // 验证序列码是否已过期
        if (licenseCodesVO.getEndUpAt().before(new Date())) {
            throw new BackendRuntimeException(VppDeviceConstant.EXCEPTION_MSG_LICENSE_EXPIRED);
        }

        // 验证序列码的有效标志
        if (!licenseCodesVO.getValidFlag().equals(LicenseCodeStatusEnum.PARTIALLY_ACTIVATED.code())
                && !licenseCodesVO.getValidFlag().equals(LicenseCodeStatusEnum.ASSIGNED.code())) {
            throw new BackendRuntimeException(VppDeviceConstant.EXCEPTION_MSG_INVALID_LICENSE_KEY);
        }

        // 查询设备状态列表
        DevicesExample devicesExample1 = new DevicesExample();
        devicesExample1.createCriteria()
                .andUniqueSnEqualTo(verifyParam.getDevice().getUniqueSn())
                .andAppIdEqualTo(licenseCodesVO.getAppId())
                .andStatusIn(new ArrayList<>(activeStatuses));
        List<Devices> statusDevicelist = extVppDeviceMapper.selectByExample(devicesExample1);

        if (CollectionUtils.isEmpty(statusDevicelist)) {
            // 新增设备
            Devices devices = new Devices();
            devices.setAppId(licenseCodesVO.getAppId());
            devices.setEndUpAt(licenseCodesVO.getEndUpAt());
            devices.setMemberId(licenseCodesVO.getMemberId());
            PlatformEnum platformEnum = PlatformEnum.getEnumByCode(verifyParam.getDevice().getPlatform());
            devices.setPlatform(platformEnum.code());
            devices.setSubscriptionId(licenseCodesVO.getSubscriptionId());
            devices.setStatus(DeviceStatusEnum.ACTIVE.code());
            devices.setCdkey(licenseCodesVO.getCdkey());
            devices.setActivatedDate(new Date());
            devices.setIsVpp(Short.valueOf("2"));
            extVppDeviceMapper.insert(devices);
        }

        // 更新序列码的使用次数和有效标志
        LicenseCodes updateCode = new LicenseCodes();
        updateCode.setUsedTimes((short) (licenseCodesVO.getUsedTimes() + 1));
        if (updateCode.getUsedTimes() < licenseCodesVO.getTimes()) {
            updateCode.setValidFlag(LicenseCodeStatusEnum.PARTIALLY_ACTIVATED.code());
        }
        if (updateCode.getUsedTimes().equals(licenseCodesVO.getTimes())) {
            updateCode.setValidFlag(LicenseCodeStatusEnum.ACTIVE.code());
        }
        licenseCodeService.updateByPrimaryKey(updateCode);

        // 返回设备信息
        DeviceResVO deviceResVO = activationVO.getDevice();
        deviceResVO.setStatus(Objects.requireNonNull(DeviceStatusEnum.getEnumByCode(statusDevicelist.get(0).getStatus())).value());
        Products products = productService.getById(licenseCodesVO.getProductId());
        deviceResVO.setProductCode(products.getCode());
        activationVO.setDevice(deviceResVO);
        return activationVO;
    }

    @Override
    public void expireDevice(String subscriptionId) {
        Devices devices = new Devices();
        devices.setStatus(DeviceStatusEnum.EXPIRED.code());
        //更新一批订阅底下的所有设备为过期
        DevicesExample devicesExample = new DevicesExample();
        devicesExample.createCriteria().andSubscriptionIdEqualTo(subscriptionId);
        extVppDeviceMapper.updateByExampleSelective(devices,devicesExample);
    }

    @Override
    public ActivationVO verify(VerifyParam verifyParam) {
        // 查询设备是否已激活
        DevicesExample devicesExample = new DevicesExample();
        devicesExample.createCriteria()
                .andUniqueSnEqualTo(verifyParam.getDevice().getUniqueSn());
        List<Devices> list = extVppDeviceMapper.selectByExample(devicesExample);
        Devices devices = new Devices();
        DeviceResVO deviceResVO = new DeviceResVO();
        if(!CollectionUtils.isEmpty(list)){
            devices.setAppVersion(verifyParam.getDevice().getAppVersion());
            devices.setId(list.get(0).getId());
            extVppDeviceMapper.updateByPrimaryKeySelective(devices);
        }else{
            // 新增设备
            AppsExample example = new AppsExample();
            example.createCriteria().andCodeEqualTo(verifyParam.getSubscription().getAppCode());
            List<Apps> apps = appsMapper.selectByExample(example);
            if (!CollectionUtils.isEmpty(apps)) {
                devices.setAppId(apps.get(0).getId());
            }
            devices.setAppVersion(verifyParam.getDevice().getAppVersion());
            devices.setModel(verifyParam.getDevice().getModel());
            devices.setOs(verifyParam.getDevice().getOs());
            devices.setUniqueSn(verifyParam.getDevice().getUniqueSn());
            devices.setTimeZone(verifyParam.getDevice().getTimeZone());
            PlatformEnum platformEnum = PlatformEnum.getEnumByCode(verifyParam.getDevice().getPlatform());
            devices.setPlatform(platformEnum.code());
            devices.setStatus(DeviceStatusEnum.UNACTIVATED.code());
            devices.setIsVpp(Short.valueOf("2"));
            extVppDeviceMapper.insert(devices);
        }
        deviceResVO.setStatus(Objects.requireNonNull(DeviceStatusEnum.getEnumByCode(devices.getStatus())).value());
        if(list.size()>0&&list.get(0).getStatus().equals(DeviceStatusEnum.ACTIVE.code())){
            List<LicenseCodesVO> licenseCodes = licenseCodeService.getWithInfo(verifyParam.getSubscription().getCdkey());
            if (CollectionUtils.isEmpty(licenseCodes)) {
                throw new BackendRuntimeException(VppDeviceConstant.EXCEPTION_MSG_INVALID_LICENSE_KEY);
            }
            LicenseCodesVO licenseCodesVO = licenseCodes.get(0);
            Products products = productService.getById(licenseCodesVO.getProductId());
            deviceResVO.setProductCode(products.getCode());
        }
        ActivationVO activationVO = new ActivationVO();
        activationVO.setDevice(deviceResVO);
        return activationVO;
    }
}
