package cn.kdan.pdf.tech.core.service.impl;

import cn.kdan.pdf.tech.core.model.*;
import cn.kdan.pdf.tech.core.service.*;
import cn.kdan.pdf.tech.core.utils.KeyFileParseUtils;
import lombok.RequiredArgsConstructor;
import lombok.extern.slf4j.Slf4j;
import org.springframework.stereotype.Service;
import org.springframework.transaction.annotation.Transactional;
import org.springframework.web.multipart.MultipartFile;

import java.io.InputStream;
import java.util.List;

/**
 * @author ComPDFKit-WPH 2024/1/2
 */
@Service
@Slf4j
@RequiredArgsConstructor
public class VppKeyFileServiceImpl implements VppKeyFileService {

    private final VppCompanyService vppCompanyService;
    private final VppMemberService vppMemberService;
    private final VppTeamService vppTeamService;
    private final VppRTeamMemberRoleService vppRTeamMemberRoleService;
    private final OrderService orderService;
    private final SubscriptionService subscriptionService;
    private final VppLicenseCodeService licenseService;

    @Transactional
    @Override
    public void parseKeyFile(MultipartFile keyFile, boolean updateOrInit) {
        KeyFileParseUtils.FileContent fileContent;
        try (InputStream inputStream = keyFile.getInputStream()) {
            fileContent = KeyFileParseUtils.parseKeyFile(inputStream, updateOrInit);
        } catch (Exception e) {
            log.error("解析key文件失败, ".concat(e.getMessage()), e);
            throw new RuntimeException("The key file signature is invalid.");
        }
        if (!updateOrInit) {
            VppCompany vppCompany = fileContent.getVppCompany();
            VppMember vppMember = fileContent.getVppMember();
            VppTeam vppTeam = fileContent.getVppTeam();
            VppRTeamMemberRole vppRTeamMemberRole = fileContent.getVppRTeamMemberRole();

            vppCompanyService.insert(vppCompany);
            vppMemberService.insert(vppMember);
            vppTeamService.insert(vppTeam);
            vppRTeamMemberRoleService.insert(vppRTeamMemberRole);
        }

        Orders order = fileContent.getOrders();
        Subscriptions subscription = fileContent.getSubscriptions();
        List<? extends LicenseCodes> licenseCodes = fileContent.getLicenseCodesList();
        orderService.insert(order);
        subscriptionService.insert(subscription);

        for (LicenseCodes licenseCode : licenseCodes) {
            licenseService.insert(licenseCode);
        }
    }

    @Override
    public Boolean updateOrInit() {
        return vppCompanyService.count() > 0;
    }

}
